#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "libnfs.h"

int main(int argc, char **argv) {

	for(;;) {
		char input[1024];
		char *buf=input;
		int count=0;
		char *args[512];

		fprintf(stdout, "> ");
		fflush(stdout);
		fgets(input, 1024, stdin);
		
		while(buf[0]!='\0') {
			args[count]=strtok_r(NULL, " \t\n", &buf);
			count++;
		}

		if(count<1) {
			continue;
		} else if(strcmp(args[0], "exit")==0 || strcmp(args[0], "quit")==0) {
			break;
		} else if(strcmp(args[0], "ls")==0 && count==1) {
			nfsDIR *dp=nfsOpendir(".");
			if(dp==NULL) {
				continue;
			}
			for(;;) {
				char *file=nfsReaddir(dp);
				if(file==NULL) {
					break;
				}
				printf("%s\n", file);
			}
			nfsClosedir(dp);
		} else if(strcmp(args[0], "setuid")==0 && count==1) {
			nfsSetuid(getuid());
		} else if(count<2) {
			continue;
		} else if(strcmp(args[0], "ls")==0) {
			nfsDIR *dp=nfsOpendir(args[1]);
			if(dp==NULL) {
				continue;
			}
			for(;;) {
				char *file=nfsReaddir(dp);
				if(file==NULL) {
					break;
				}
				printf("%s\n", file);
			}
			nfsClosedir(dp);

		} else if(strcmp(args[0], "umount")==0) {
			nfsUmount(args[1]);
		} else if(strcmp(args[0], "setuid")==0) {
			nfsSetuid(atoi(args[1]));
		} else if(strcmp(args[0], "cd")==0) {
			nfsChdir(args[1]);
		} else if(strcmp(args[0], "rm")==0) {
			nfsUnlink(args[1]);
		} else if(strcmp(args[0], "rmdir")==0) {
			nfsRmdir(args[1]);
		} else if(strcmp(args[0], "touch")==0) {
			nfsCreate(args[1], 0666);
		} else if(strcmp(args[0], "mkdir")==0) {
			nfsMkdir(args[1], 0777);
		} else if(strcmp(args[0], "readlink")==0) {
			char buf[1024];
			if(nfsReadlink(args[1], buf, 1024)>=0) {
				printf("%s\n", buf);
			}
		} else if(strcmp(args[0], "stat")==0) {
			struct stat info;
			if(nfsStat(args[1], &info)==-1) {
				continue;
			}
			printf("Mode: 0%o\n", info.st_mode);
			printf("Size: %lu bytes\n", info.st_size);
			printf("Owner: %d\n", info.st_uid);
			printf("Group: %d\n", info.st_gid);
			printf("Links: %d\n", info.st_nlink);
			printf("Access Time: %ld\n", info.st_atime);
			printf("Modification Time: %ld\n", info.st_mtime);
			printf("Creation Time: %ld\n", info.st_ctime);
		} else if(strcmp(args[0], "cat")==0) {
			nfsFILE *fp=nfsOpen(args[1], O_RDONLY);
			if(fp==NULL) {
				continue;
			}
			for(;;) {
				char buf[1024];
				ssize_t bytes;

				bytes=nfsRead(fp, buf, 1024);
				if(bytes<=0) {
					break;
				}
				write(1, buf, bytes);
			}
			nfsClose(fp);
		} else if(strcmp(args[0], "get")==0) {
			int fd;
			nfsFILE *fp=nfsOpen(args[1], O_RDONLY);
			if(fp==NULL) {
				continue;
			}
			fd=open(args[1], O_WRONLY | O_CREAT | O_EXCL, 0666);
			if(fd==-1) {
				perror("open");
				nfsClose(fp);
				continue;
			}
			for(;;) {
				char buf[1024];
				ssize_t bytes;

				bytes=nfsRead(fp, buf, 1024);
				if(bytes<=0) {
					break;
				}
				write(fd, buf, bytes);
			}
			nfsClose(fp);
			close(fd);
		} else if(strcmp(args[0], "put")==0) {
			int fd;
			nfsFILE *fp;
			if(nfsCreate(args[1], 0666)!=0) {
				continue;
			}
			fp=nfsOpen(args[1], O_WRONLY);
			if(fp==NULL) {
				continue;
			}
			fd=open(args[1], O_RDONLY);
			for(;;) {
				char buf[1024];
				ssize_t bytes;

				bytes=read(fd, buf, 1024);
				if(bytes<=0) {
					break;
				}
				nfsWrite(fp, buf, bytes);
			}
			nfsClose(fp);
			close(fd);
		} else if(count<3) {
			continue;
		} else if(strcmp(args[0], "mount")==0) {
			nfsMount(args[1], args[2]);
		} else if(strcmp(args[0], "ln")==0 && count==3) {
			nfsLink(args[1], args[2]);
		} else if(strcmp(args[0], "mv")==0) {
			nfsRename(args[1], args[2]);
		} else if(strcmp(args[0], "chown")==0) {
			nfsChown(args[2], atoi(args[1]));
		} else if(strcmp(args[0], "chgrp")==0) {
			nfsChgrp(args[2], atoi(args[1]));
		} else if(strcmp(args[0], "chmod")==0) {
			nfsChmod(args[2], strtol(args[1], (char **)NULL, 8));
		} else if(count<4) {
			continue;
		} else if(strcmp(args[0], "ln")==0 && strcmp(args[1], "-s")==0) {
			nfsSymlink(args[2], args[3]);
		}
	}
	return 0;
}
